package com.supervision.contoller;

import com.supervision.dto.R;
import com.supervision.dto.robot.RobotTalkDTO;
import com.supervision.model.RobotTalkReq;
import com.supervision.service.DifyService;
import com.supervision.service.IChatService;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import reactor.core.publisher.Flux;

import java.io.IOException;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/chat")
@RequiredArgsConstructor
public class ChatController {

    private final IChatService chatService;
    private final DifyService difyService;

    @PostMapping("/talk")
    public R<RobotTalkDTO> talk(@RequestParam("file") MultipartFile multipartFile, @ModelAttribute RobotTalkReq robotTalkReq) {
        RobotTalkDTO talk = chatService.talk(multipartFile,robotTalkReq);
        return R.ok(talk);
    }

    @PostMapping("/asr")
    public R<String> asr(@RequestParam("file") MultipartFile file) throws IOException {
        return R.ok(chatService.asr(file));
    }

    @GetMapping("/talkList")
    public R<List<RobotTalkDTO>> talkList(@RequestParam("sessionId") String sessionId) {
        List<RobotTalkDTO> commonDialogVos = chatService.talkList(sessionId);
        return R.ok(commonDialogVos);
    }

    @GetMapping("/getAudio")
    public void getAudio(HttpServletResponse response,
                         @RequestParam("audioId")String audioId) throws IOException {
         chatService.getAudio(response,audioId);
    }

    @GetMapping(value="/stream",produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<ServerSentEvent<Map<String, String>>> stream(@RequestParam("query") String query,@RequestParam("sessionId") String sessionId) {
        return chatService.streamingMessage(query,sessionId);
    }
}
