package com.supervision;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.supervision.dto.dify.ChatResDTO;
import com.supervision.dto.paddlespeech.res.TtsResultDTO;
import com.supervision.dto.robot.AnswerInfo;
import com.supervision.dto.robot.AskInfo;
import com.supervision.model.dify.DIFYChatReqInputVO;
import com.supervision.model.dify.DifyChatReqVO;
import com.supervision.model.dify.StreamResponse;
import com.supervision.util.AsrUtil;
import com.supervision.util.DifyApiUtil;
import com.supervision.util.TtsUtil;
import com.supervision.util.WavUtil;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@SpringBootTest
class SpeechDemoServiceApplicationTests {

    @Test
    void contextLoads() {
    }


    @Test
    void testTtsTransform() {

        String text = """
                您好，我是您的康养顾问小苏，很开心接到您的来电
                您好，我是您的康养顾问小苏，感谢您的来电
                您好，我是您的康养顾问小陈，很高兴接到您的电话
                关于您的问题，我已经为您整理好了相关信息
                我会结合最新的资料，为您提供最贴心的解答
                结合最新的信息，为您提供最合适的建议
                结合最新的资料为您提供详细的解答
                根据掌握的高黎贡勐赫小镇信息，为您提供最准确的服务与建议
                根据最新动态，为您提供最精准的解答与建议
                结合最新情况，为您提供最贴心的服务与支持
                """;

        List<String> collect = Arrays.stream(text.split("\n")).filter(s -> !s.isBlank()).toList();
        for (int i = 0; i < collect.size(); i++) {
            TtsResultDTO ttsResultDTO = TtsUtil.ttsTransform(collect.get(i));
            String jsonStr = JSONUtil.toJsonStr(ttsResultDTO);
            System.out.println(jsonStr);
            // https://www.toolfk.com/zh-cn/tools/base64-to-audio.html base64转音频
           /* byte[] decode = Base64.decode(ttsResultDTO.getAudio());
            FileUtil.writeBytes(decode, "F:\\tmp\\audio1\\tts" + i + ".wav");*/
        }

    }

    @Value("${paddle-speech.text:xx}")
    private String voiceBase64;
    @Test
    void testAsrTransform2() {

        String text = AsrUtil.asrTransformByBytes(voiceBase64);
        System.out.println(text);

    }


    @Resource
    private DifyApiUtil difyApiUtil;
    @Test
    public void chatTest() {
        DifyChatReqVO difyChatReqVO = new DifyChatReqVO();
        difyChatReqVO.setUser("admin");
        DIFYChatReqInputVO inputs = new DIFYChatReqInputVO();
        difyChatReqVO.setQuery("我叫什么？");
        difyChatReqVO.setInputs(inputs);

        System.out.println(JSONUtil.toJsonStr(difyChatReqVO));
        ChatResDTO chat = difyApiUtil.chat(difyChatReqVO);
        System.out.println(JSONUtil.toJsonStr(chat));
    }

    @Autowired
    WebClient webClient;
    @Value("${dify.url}")
    private String difyUrl;

    @Value("${dify.app-auth}")
    private String difyAppAuth;
    @Test
    public void streamChatTest() {

        DifyChatReqVO difyChatReqVO = new DifyChatReqVO();
        difyChatReqVO.setUser("admin");
        DIFYChatReqInputVO inputs = new DIFYChatReqInputVO();
        difyChatReqVO.setQuery("我叫什么？");
        //difyChatReqVO.setConversation_id("123");
        difyChatReqVO.setInputs(inputs);

        StringBuilder fullAskString = new StringBuilder();
        List<String> audioList = new ArrayList<>();
        StringBuilder sentence = new StringBuilder();

        webClient.post()
                .uri(difyUrl)
                .headers(httpHeaders -> {
                    httpHeaders.setContentType(MediaType.APPLICATION_JSON);
                    httpHeaders.setBearerAuth(difyAppAuth);
                })
                .bodyValue(JSON.toJSONString(difyChatReqVO))
                .retrieve()
                .bodyToFlux(StreamResponse.class)
                .map(response -> {

                    log.info("response: {}", response);

                    Map<String, String> map = new HashMap<>();
                    map.put("event", response.getEvent());
                    if (response.getEvent().equals("message") && response.getAnswer() != null) {
                        //遍历answer中的每一个字符，判断是否为标点符号，如果是，说明是句子的结尾，将标点符号前的文本拼接到sentence中，并打印，然后清空sentence，如果标点符号后还有文本，将文本拼接到sentence中
                        for (char ch : response.getAnswer().toCharArray()) {
                            sentence.append(ch);
                            fullAskString.append(ch);
                            if (ch == '。' || ch == '！' || ch == '？' || ch == '，' || ch == '、' || ch == '‘' || ch == '’' || ch == '“' || ch == '”') { // Check for punctuation marks
                                log.info(sentence.toString());
                                TtsResultDTO ttsResultDTO = TtsUtil.ttsTransform(sentence.toString());
                                String voiceBaseId = UUID.randomUUID().toString();
                                audioList.add(voiceBaseId);
                                map.put("audioId", voiceBaseId);
                                sentence.setLength(0);
                            }
                        }
                    }
                    if (response.getEvent().equals("message_end")) {
                        map.put("sessionId", response.getConversation_id());
                        if (!sentence.isEmpty()) {
                            log.info(sentence.toString());
                            TtsResultDTO ttsResultDTO = TtsUtil.ttsTransform(sentence.toString());
                            String voiceBaseId = UUID.randomUUID().toString();
                            audioList.add(voiceBaseId);
                            map.put("audioId", voiceBaseId);
                        }
                    }
                    return null;
                }).filter(Objects::nonNull);


        while (true){
            log.info("sleep....");
            ThreadUtil.sleep(1000);
        }
    }
}
